@@ -32,7 +32,7 @@
 #include <libgnomevfs/gnome-vfs-module-callback-module-api.h>
 #include <libgnomevfs/gnome-vfs-standard-callbacks.h>
 
-#include <librio500_api.h>
+#include <rio500/librio500_api.h>
 
 typedef struct _RioTreeContent RioTreeContent;
 
@@ -66,8 +66,45 @@ static RioTreeContent *rio = NULL;
 #define UNLOCK_RIO() 	g_mutex_unlock (rio_lock)
 #endif
 
-static RioTreeContent* update_tree_content(RioTreeContent *tmp);
-static void destroy_rio(RioTreeContent *tmp);
+static void update_tree_content ();
+static void destroy_rio ();
+static const char *gnome_vfs_uri_get_basename (const GnomeVFSURI *uri);
+
+const char *
+gnome_vfs_uri_get_basename (const GnomeVFSURI *uri)
+{
+	char *p;
+
+	g_return_val_if_fail (uri != NULL, NULL);
+
+	if (uri->text == NULL) {
+		return NULL;
+	}
+
+	p = strrchr (uri->text, GNOME_VFS_URI_PATH_CHR);
+	if (p == NULL) {
+		return NULL;
+	}
+
+	p++;
+	if (*p == '\0') {
+		return NULL;
+	}
+
+	return p;
+}
+
+static GList *
+rio_get_content_from_card (int card)
+{
+	if (card == 0)
+		return rio->int_content;
+
+	if ((card == 1) && (rio->have_flash == TRUE))
+		return rio->ext_content;
+
+	return NULL;
+}
 
 static GnomeVFSResult
 lookup_uri(GnomeVFSURI *uri, gint *card, gint *folder, gint *song)
@@ -83,7 +120,7 @@ lookup_uri(GnomeVFSURI *uri, gint *card, gint *folder, gint *song)
 
 	DEBUG_RIO(("lookup_uri: %s\n", gnome_vfs_uri_to_string(uri, 0)));
 
-	rio = update_tree_content(rio);
+	update_tree_content();
 
 	LOCK_RIO();
 	if (rio == NULL)
@@ -221,24 +258,24 @@ lookup_uri(GnomeVFSURI *uri, gint *card, gint *folder, gint *song)
 	return GNOME_VFS_OK;
 }
 
-static RioTreeContent*
-update_tree_content(RioTreeContent *tmp)
+static void
+update_tree_content()
 {
 	RioFolderEntry *folder_entry = NULL;
 	gint card_num, major, minor;
 
 	DEBUG_RIO(("update_tree_content start\n"));
 
-	if (tmp != NULL)
+	if (rio != NULL)
 	{
 		DEBUG_RIO(("update_tree_content: RioTreeContent is not NULL\n"));
-		if (tmp->dirty == TRUE)
+		if (rio->dirty == TRUE)
 		{
 			DEBUG_RIO(("update_tree_content: RioTreeContent is dirty\n"));
-			destroy_rio(tmp);
+			destroy_rio(rio);
 		} else {
 			DEBUG_RIO(("update_tree_content: RioTreeContent is not dirty\n"));
-			return rio;
+			return;
 		}
 	}
 
@@ -246,36 +283,37 @@ update_tree_content(RioTreeContent *tmp)
 
 	LOCK_RIO();
 
-	tmp = g_new(RioTreeContent, 1);
-	tmp->rio_hw = NULL;
-	tmp->int_content = NULL;
-	tmp->ext_content = NULL;
-	tmp->virt_content = NULL;
-	tmp->int_mem_total = tmp->ext_mem_total = tmp->int_mem_free =
-		tmp->ext_mem_free = 0;
-	tmp->dirty = FALSE;
+	rio = g_new(RioTreeContent, 1);
+	rio->rio_hw = NULL;
+	rio->int_content = NULL;
+	rio->ext_content = NULL;
+	rio->virt_content = NULL;
+	rio->int_mem_total = rio->ext_mem_total = rio->int_mem_free =
+		rio->ext_mem_free = 0;
+	rio->dirty = FALSE;
 
-	tmp->rio_hw = rio_new();
-	if (rio_check(tmp->rio_hw) == FALSE)
+	rio->rio_hw = rio_new();
+	if (rio_check (rio->rio_hw) == FALSE)
 	{
 		DEBUG_RIO(("update_tree_content: check of rio is FALSE, destroying\n"));
-		destroy_rio(tmp);
+		destroy_rio (rio);
+		rio = NULL;
 		UNLOCK_RIO();
-		return NULL;
+		return;
 	}
 
-	rio_get_revision(tmp->rio_hw, &major, &minor);
+	rio_get_revision (rio->rio_hw, &major, &minor);
 
 	DEBUG_RIO(("update_tree_content: version %d.%d\n", major, minor));
 	
-	card_num = rio_get_card_number(tmp->rio_hw);
+	card_num = rio_get_card_number (rio->rio_hw);
 	if (card_num >= 2)
-		tmp->have_flash = TRUE;
+		rio->have_flash = TRUE;
 
-	rio_set_card(tmp->rio_hw, 0);
-	tmp->int_mem_free = rio_memory_left(tmp->rio_hw);
-	tmp->int_mem_total = rio_get_mem_total(tmp->rio_hw);
-	tmp->int_content = rio_get_content(tmp->rio_hw);
+	rio_set_card (rio->rio_hw, 0);
+	rio->int_mem_free = rio_memory_left (rio->rio_hw);
+	rio->int_mem_total = rio_get_mem_total (rio->rio_hw);
+	rio->int_content = rio_get_content (rio->rio_hw);
 
 	DEBUG_RIO(("update_tree_content: got internal card info\n"));
 
@@ -283,20 +321,20 @@ update_tree_content(RioTreeContent *tmp)
 	folder_entry->name = g_strdup("Internal");
 	folder_entry->folder_num = 0;
 	folder_entry->songs = NULL;
-	tmp->virt_content = g_list_append(tmp->virt_content,
+	rio->virt_content = g_list_append(rio->virt_content,
 			(gpointer) folder_entry);
 
 	DEBUG_RIO(("update_tree_content: created dummy folder for internal\n"));
 
-	if (tmp->have_flash == TRUE)
+	if (rio->have_flash == TRUE)
 	{
-		rio_set_card(tmp->rio_hw, 1);
-		tmp->ext_mem_free = rio_memory_left(tmp->rio_hw);
-		tmp->ext_mem_total = rio_get_mem_total(tmp->rio_hw);
+		rio_set_card(rio->rio_hw, 1);
+		rio->ext_mem_free = rio_memory_left(rio->rio_hw);
+		rio->ext_mem_total = rio_get_mem_total(rio->rio_hw);
 
 		DEBUG_RIO(("update_tree_content: getting external card info\n"));
 		
-		tmp->ext_content = rio_get_content(tmp->rio_hw);
+		rio->ext_content = rio_get_content(rio->rio_hw);
 
 		DEBUG_RIO(("update_tree_content: got external card info\n"));
 
@@ -304,35 +342,36 @@ update_tree_content(RioTreeContent *tmp)
 		folder_entry->name = g_strdup("External");
 		folder_entry->folder_num = 1;
 		folder_entry->songs = NULL;
-		tmp->virt_content = g_list_append(tmp->virt_content,
+		rio->virt_content = g_list_append(rio->virt_content,
 				(gpointer) folder_entry);
 
 		DEBUG_RIO(("update_tree_content: created dummy folder for external\n"));
 	}
-	rio_delete(tmp->rio_hw);
+	rio_delete(rio->rio_hw);
 	UNLOCK_RIO();
 
-	return tmp;
+	return;
 }
 
 static void
-destroy_rio(RioTreeContent *tmp)
+destroy_rio()
 {
-	g_return_if_fail(tmp != NULL);
+	g_return_if_fail(rio != NULL);
 
 	DEBUG_RIO(("destroy_rio: start\n"));
-	if (tmp->int_content != NULL)
-		rio_destroy_content(tmp->int_content);
+	if (rio->int_content != NULL)
+		rio_destroy_content(rio->int_content);
 	DEBUG_RIO(("destroy_rio: destroyed int_content\n"));
-	if (tmp->have_flash == TRUE
-			&& tmp->ext_content != NULL)
-		rio_destroy_content(tmp->ext_content);
+	if (rio->have_flash == TRUE
+			&& rio->ext_content != NULL)
+		rio_destroy_content(rio->ext_content);
 	DEBUG_RIO(("destroy_rio: destroyed ext_content\n"));
-	if (tmp->virt_content != NULL)
-		rio_destroy_content(tmp->virt_content);
+	if (rio->virt_content != NULL)
+		rio_destroy_content(rio->virt_content);
 	DEBUG_RIO(("destroy_rio: destroyed virt_content\n"));
 
-	g_free(tmp);
+	g_free(rio);
+	rio = NULL;
 
 	DEBUG_RIO(("destroy_rio: end\n"));
 }
@@ -365,18 +404,13 @@ rio_to_vfs_error(gint error)
 }
 
 typedef struct {
+	/* is it a read or a write to the rio500 */
 	gboolean is_read;
-	GnomeVFSFileOffset offset;
-	/* For reading */
-	GList *folders, *songs;
-	folder_entry *f_entry;
-	song_entry *s_entry;
-	gint old_offset, folder_block_offset;
-	gint folder;
-	unsigned long size;
-	gboolean init_done;
+	/* handle for the temp file */
+	GnomeVFSHandle *handle;
 	/* For writing */
-	//FIXME
+	gchar *filename;
+	gint folder;
 } FileHandle;
 
 static GnomeVFSResult
@@ -393,9 +427,10 @@ do_open (GnomeVFSMethod *method,
 
 	DEBUG_RIO(("do_open() %s mode %d\n",
 				gnome_vfs_uri_to_string (uri, 0), mode));
-//FIXME
-	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 
+	if (g_file_test ("/tmp/rio500_tmp_file.mp3", G_FILE_TEST_EXISTS)
+			== TRUE)
+		return GNOME_VFS_ERROR_TOO_MANY_OPEN_FILES;
 
 	if (mode & GNOME_VFS_OPEN_READ)
 		is_read = TRUE;
@@ -413,22 +448,24 @@ do_open (GnomeVFSMethod *method,
 	if ((res != GNOME_VFS_OK) && (is_read == TRUE))
 	{
 		UNLOCK_RIO();
+		DEBUG_RIO(("do_open(): file not found on read open, result\n"));
 		return res;
 	}
 
-	if ((folder != -1) && (song == -1))
+	if ((folder != -1) && (song == -1) && (is_read == TRUE))
 	{
 		UNLOCK_RIO();
+		DEBUG_RIO(("do_open(): file is a directory on a read open\n"));
 		return GNOME_VFS_ERROR_IS_DIRECTORY;
 	}
 
 	if ((song == -1) && (is_read == TRUE))
 	{
 		UNLOCK_RIO();
+		DEBUG_RIO(("do_open(): file not found on a read open\n"));
 		return GNOME_VFS_ERROR_NOT_FOUND;
 	}
 
-	rio->rio_hw = rio_new();
 	if (rio_check(rio->rio_hw) == FALSE)
 	{
 		DEBUG_RIO(("do_open: check of rio is FALSE, destroying\n"));
@@ -439,138 +476,69 @@ do_open (GnomeVFSMethod *method,
 
 	if (is_read == TRUE)
 	{
-		GList *folders, *songs;
-		folder_entry *f_entry;
-		song_entry *s_entry;
-		gint old_offset, folder_block_offset;
+		GnomeVFSHandle *local_handle;
+		gint res;
+
 
 		DEBUG_RIO(("do_open: opening read-only\n"));
 
-		/* Init communication with rio */
-		start_comm (rio->rio_hw);
+		rio->rio_hw = rio_new();
+		res = rio_get_song(rio->rio_hw, "/tmp/rio500_tmp_file.mp3",
+				folder, song);
+		rio_delete (rio->rio_hw);
 
-		/* Read folder & song block */
-		send_command(rio->rio_hw->rio_dev, 0x42, 0, 0);
-		folders = read_folder_entries (rio->rio_hw->rio_dev,
-				rio->rio_hw->card);
-		if ( folder > g_list_length (folders)-1 )
-			folder = 0;
+		if (res != RIO_SUCCESS)
+		{
+			unlink ("/tmp/rio500_tmp_file.mp3");
+			UNLOCK_RIO();
+			DEBUG_RIO(("do_open(): error getting song from the rio\n"));
+			return GNOME_VFS_ERROR_IO;
+		}
 
-		songs = read_song_entries (rio->rio_hw->rio_dev,
-				folders, folder, rio->rio_hw->card);
+		DEBUG_RIO(("do_open: copied to /tmp/rio500_tmp_file.mp3\n"));
 
-		/* Trick the rio into reading song data */
-		f_entry = (folder_entry *) g_list_nth_data (folders, 0);
-		s_entry = (song_entry *) g_list_nth_data (songs, song);
+		if (gnome_vfs_open (&local_handle, "/tmp/rio500_tmp_file.mp3",
+				GNOME_VFS_OPEN_READ) != GNOME_VFS_OK)
+		{
+			unlink ("/tmp/rio500_tmp_file.mp3");
+			UNLOCK_RIO();
+			DEBUG_RIO(("do_open(): error opening the tmp file\n"));
+			return GNOME_VFS_ERROR_IO;
+		}
 
-		old_offset      = f_entry->offset;
-		f_entry->offset = s_entry->offset;
-
-		write_folder_entries(rio->rio_hw->rio_dev,
-				folders, rio->rio_hw->card);
-		send_command (rio->rio_hw->rio_dev, 0x42, 0, 0);
-		send_command (rio->rio_hw->rio_dev, 0x42, 0, 0);
-		folder_block_offset = send_command (rio->rio_hw->rio_dev,
-				0x43, 0x0, 0x0);
-
-		/* Tell Rio where the root folder block is. */
-		send_folder_location(rio->rio_hw->rio_dev,
-				folder_block_offset, folder,
-				rio->rio_hw->card);
-
-		/* Not really sure what this does */
-		send_command (rio->rio_hw->rio_dev, 0x58, 0x0,
-				rio->rio_hw->card);
+		DEBUG_RIO(("do_open: opened local file\n"));
 
 		/* Create the handle */
 		handle = g_new(FileHandle, 1);
 		/* global */
 		handle->is_read = is_read;
-		handle->offset = 0;
-		/* read specific */
-		handle->folders = folders;
-		handle->songs = songs;
-		handle->f_entry = f_entry;
-		handle->s_entry = s_entry;
-		handle->old_offset = old_offset;
-		handle->folder_block_offset = folder_block_offset;
-		handle->folder = folder;
-		handle->size = s_entry->length;
-		handle->init_done = FALSE;
-		handle->offset = 0;
-		DEBUG_RIO(("do_open: length: %lu\n", handle->size));
-	}
-#if 0
-	GnomeVFSResult res;
-	SmbConnection *connection;
-	SmbHandle *handle;
-	SmbVirtualFile *file;
-	char *path_remainder;
-	int fnum;
-	char *filename;
-	char *dos_filename;
-	int flags;
-	char *server;
-	char *path;
-	char *end;
-	const char *user;
-	const char *password;
-	
-	LOCK_SAMBA();
-	
-	DEBUG_SMB (("do_open() %s mode %d\n",
-		    gnome_vfs_uri_to_string (uri, 0), mode));
-	
-	if ((mode & GNOME_VFS_OPEN_READ) &&
-	    (mode & GNOME_VFS_OPEN_WRITE))
-		flags = O_RDWR;
-	else if (mode & GNOME_VFS_OPEN_READ)
-		flags = O_RDONLY;
-	else if (mode & GNOME_VFS_OPEN_WRITE)
-		flags = O_WRONLY;
-	else
-		return GNOME_VFS_ERROR_INVALID_OPEN_MODE;
+		handle->handle = local_handle;
+		handle->filename = NULL;
+	} else {
+		GnomeVFSHandle *local_handle;
+		gchar *basename, *local;
 
-	res = lookup_uri (uri, &file, &path_remainder, &connection, TRUE);
-
-	if (res) {
-		UNLOCK_SAMBA ();
-		return res;
-	}
-
-	if (file->virtual_type == SMB_VIRTUAL_TYPE_DIRECTORY) {
-		UNLOCK_SAMBA ();
-		return GNOME_VFS_ERROR_IS_DIRECTORY;
-	}
-
-	if (file->virtual_type == SMB_VIRTUAL_TYPE_FILE) {
-		handle = g_new (SmbHandle, 1);
-		handle->is_data = TRUE;
-		handle->offset = 0;
-		handle->file_data = g_strdup (file->u.file.contents);
-	} else { /* SMB_VIRTUAL_TYPE_SHARE */
-		dos_filename = unix_filename_to_dos ((path_remainder)?path_remainder:"/");
-		fnum = cli_open (connection->cli, dos_filename, flags, DENY_NONE);
-		g_free (path_remainder);
-		g_free (dos_filename);
-		if (fnum == -1) {
-			res = gnome_vfs_result_from_cli (connection->cli);
-			smb_connection_unref (connection);
-			UNLOCK_SAMBA();
-			return res;
+		basename = g_strdup (gnome_vfs_uri_get_basename (uri));
+		local = g_strdup_printf ("file://%s/%s", g_get_tmp_dir(), basename);
+		if (gnome_vfs_create (&local_handle, local,
+		    GNOME_VFS_OPEN_WRITE, FALSE, 0600) != GNOME_VFS_OK)
+		{
+			UNLOCK_RIO();
+			DEBUG_RIO(("do_open(): error opening the temp file for writing %s\n", local));
+			g_free (local);
+			return GNOME_VFS_ERROR_IO;
 		}
-		
-		handle = g_new (SmbHandle, 1);
-		handle->is_data = FALSE;
-		handle->connection = connection;
-		handle->fnum = fnum;
-		handle->offset = 0;
+
+		/* Create the handle */
+		handle = g_new(FileHandle, 1);
+		handle->is_read = is_read;
+		handle->handle = local_handle;
+		handle->folder = folder;
+		handle->filename = gnome_vfs_get_local_path_from_uri (local);
+
+		g_free (local);
 	}
 
-	UNLOCK_SAMBA();
-
-	*method_handle = (GnomeVFSMethodHandle *)handle;
-#endif
 	*method_handle = (GnomeVFSMethodHandle *)handle;
 	return GNOME_VFS_OK;
 }
@@ -582,56 +550,47 @@ do_close (GnomeVFSMethod *method,
 
 {
 	FileHandle *handle = (FileHandle *)method_handle;
+	GnomeVFSResult res = GNOME_VFS_OK;
+
+	DEBUG_RIO(("do_close(): is_read: %d\n", handle->is_read));
 
 	if (handle->is_read == TRUE)
 	{
-		/* Restore folder */
-		handle->f_entry->offset = handle->old_offset;
-		write_folder_entries(rio->rio_hw->rio_dev, handle->folders,
-				rio->rio_hw->card);
-		send_command (rio->rio_hw->rio_dev, 0x42, 0, 0);
-		send_command (rio->rio_hw->rio_dev, 0x42, 0, 0);
-		handle->folder_block_offset = send_command
-			(rio->rio_hw->rio_dev, 0x43, 0x0, 0x0);
+		gint tmp;
+		
+		DEBUG_RIO(("do_close(): unlinking temp file\n"));
+		tmp = unlink ("/tmp/rio500_tmp_file.mp3");
+		if (tmp != 0)
+			res = GNOME_VFS_ERROR_IO;
+		else
+			res = GNOME_VFS_OK;
+	} else {
+		gint tmp;
 
-		/* Tell Rio where the root folder block is. */
-		send_folder_location(rio->rio_hw->rio_dev,
-				handle->folder_block_offset,
-				handle->folder, rio->rio_hw->card);
+		DEBUG_RIO(("do_close(): uploading song %s to folder %d\n",
+					handle->filename, handle->folder));
 
-		/* Not really sure what this does */
-		send_command (rio->rio_hw->rio_dev, 0x58, 0x0,
-				rio->rio_hw->card);
+		rio->rio_hw = rio_new();
+		tmp = rio_add_song (rio->rio_hw, handle->folder,
+				handle->filename);
+		rio_delete(rio->rio_hw);
+
+		rio->dirty = TRUE;
+
+		DEBUG_RIO(("do_close(): uploading result: %d\n", res));
+
+		unlink (handle->filename);
+		res = rio_to_vfs_error(tmp);
 	}
 
-	/* Close communication */
-	end_comm (rio->rio_hw);
-
-	rio_delete(rio->rio_hw);
-
-	//FIXME delete the handle's content
+	/* delete the handle's content */
+	gnome_vfs_close (handle->handle);
+	g_free (handle->filename);
+	g_free (handle);
 
 	UNLOCK_RIO();
 
-	return GNOME_VFS_OK;
-#if 0
-	SmbHandle *handle = (SmbHandle *)method_handle;
-	size_t n;
-
-	LOCK_SAMBA();
-	DEBUG_SMB (("do_close() %p\n", method_handle));
-
-	if (handle->is_data) {
-		g_free (handle->file_data);
-	} else {
-		cli_close  (handle->connection->cli, handle->fnum);
-		smb_connection_unref (handle->connection);
-	}
-	
-	g_free (handle);
-	
-	UNLOCK_SAMBA();
-#endif
+	return res;
 }
 
 static GnomeVFSResult
@@ -642,104 +601,11 @@ do_read (GnomeVFSMethod *method,
 	 GnomeVFSFileSize *bytes_read,
 	 GnomeVFSContext *context)
 {
-	static BYTE *init_buffer = NULL;
 	FileHandle *handle = (FileHandle *)method_handle;
-	gulong chunk_size, read = 0;
 
-	DEBUG_RIO(("do_read() %Lu bytes, size %lu\n", num_bytes, handle->size));
+	DEBUG_RIO(("do_read() %Lu bytes\n", num_bytes));
 
-	if (handle->init_done == FALSE)
-	{
-		if (init_buffer == NULL)
-		{
-			gulong this_read;
-
-			/* Read 0x4000 bytes first */
-			this_read = MIN (handle->size, 0x4000);
-			init_buffer = (char *)malloc (this_read);
-
-			send_command (rio->rio_hw->rio_dev, 0x4e, 0xff, rio->rio_hw->card);
-			send_command (rio->rio_hw->rio_dev, 0x45, 0x0, this_read);
-
-			read = bulk_read (rio->rio_hw->rio_dev, init_buffer, this_read);
-			if (read > num_bytes)
-				read = num_bytes;
-
-			memcpy (buffer, init_buffer, read);
-		} else {
-			unsigned long n;
-
-			n = MIN (num_bytes,
-				MIN(handle->size, 0x4000) - handle->offset);
-			memcpy (buffer, init_buffer, n);
-			read = n;
-		}
-		handle->offset += read;
-		DEBUG_RIO(("do_read() init read %lu bytes (offset %Ld)\n", read,
-					handle->offset));
-		if (handle->offset == MIN(handle->size, 0x4000))
-			handle->init_done = TRUE;
-
-		return GNOME_VFS_OK;
-	}
-
-	if (num_bytes+handle->offset > handle->size)
-		chunk_size = handle->size - handle->offset;
-	else
-		chunk_size = num_bytes;
-	send_command (rio->rio_hw->rio_dev, 0x45, chunk_size, 0x0);
-	read = bulk_read (rio->rio_hw->rio_dev, (BYTE *)buffer,
-			chunk_size);
-	send_command (rio->rio_hw->rio_dev, 0x42, 0, 0);
-	send_command (rio->rio_hw->rio_dev, 0x42, 0, 0);
-	handle->offset += read;
-
-	*bytes_read = read;
-
-	if (handle->offset >= handle->size)
-		return GNOME_VFS_ERROR_EOF;
-	
-	DEBUG_RIO(("do_read() read %d bytes (offset %Ld)\n", *bytes_read,
-				handle->offset));
-#if 0
-	SmbHandle *handle = (SmbHandle *)method_handle;
-	size_t n;
-	int link_len;
-
-	LOCK_SAMBA();
-	DEBUG_SMB (("do_read() %p\n", method_handle));
-
-	if (handle->is_data) {
-		link_len = strlen (handle->file_data);
-		if (handle->offset >= link_len)
-			n = 0;
-		else {
-			n = MIN (num_bytes, link_len - handle->offset);
-			memcpy (buffer, handle->file_data + handle->offset, n);
-		}
-	} else {
-		n = cli_read (handle->connection->cli,
-			      handle->fnum,
-			      buffer,
-			      handle->offset,
-			      num_bytes);
-	}
-
-	UNLOCK_SAMBA();
-
-	if (n < 0) {
-		*bytes_read = 0;
-		return gnome_vfs_result_from_cli (handle->connection->cli);
-	}
-	
-	*bytes_read = n;
-	
-	if (n == 0)
-		return GNOME_VFS_ERROR_EOF;
-	
-	handle->offset += n;
-#endif
-	return GNOME_VFS_OK;
+	return gnome_vfs_read (handle->handle, buffer, num_bytes, bytes_read);
 }
 
 static GnomeVFSResult
@@ -752,40 +618,12 @@ do_write (GnomeVFSMethod *method,
 
 
 {
-#if 0
-	SmbHandle *handle = (SmbHandle *)method_handle;
-	ssize_t n;
-	int link_len;
+	FileHandle *handle = (FileHandle *)method_handle;
 
-	LOCK_SAMBA();
-	DEBUG_SMB (("do_write() %p\n", method_handle));
+	DEBUG_RIO (("do_write() %p\n", method_handle));
 
-	if (handle->is_data) {
-	  return GNOME_VFS_ERROR_NOT_SUPPORTED;
-	} else {
-		n = cli_write (handle->connection->cli,
-			       handle->fnum,
-			       0,
-			       (char *)buffer,
-			       handle->offset,
-			       num_bytes);
-	}
-
-	UNLOCK_SAMBA();
-
-	if (n < 0) {
-		*bytes_written = 0;
-		return gnome_vfs_result_from_cli (handle->connection->cli);
-	}
-	
-	*bytes_written = n;
-	
-	if (n == 0)
-		return GNOME_VFS_ERROR_EOF;
-	
-	handle->offset += n;
-#endif
-	return GNOME_VFS_OK;
+	return gnome_vfs_write (handle->handle, buffer, num_bytes,
+			bytes_written);
 }
 
 static GnomeVFSResult
@@ -798,81 +636,12 @@ do_create (GnomeVFSMethod *method,
 	   GnomeVFSContext *context)
 
 {
-#if 0
-	GnomeVFSResult res;
-	SmbVirtualFile *file;
-	SmbConnection *connection;
-	SmbHandle *handle;
-	int fnum;
-	char *path_remainder;
-	char *dos_filename;
-	int flags;
-	char *server;
-	char *path;
-
-	LOCK_SAMBA();
-	
-	DEBUG_SMB (("do_create() %s mode %d\n",
-		    gnome_vfs_uri_to_string (uri, 0), mode));
-	
-	if ((mode & GNOME_VFS_OPEN_READ) &&
-	    (mode & GNOME_VFS_OPEN_WRITE))
-		flags = O_RDWR;
-	else if (mode & GNOME_VFS_OPEN_READ)
-		flags = O_RDONLY;
-	else if (mode & GNOME_VFS_OPEN_WRITE)
-		flags = O_WRONLY;
-	else
-		return GNOME_VFS_ERROR_INVALID_OPEN_MODE;
+	DEBUG_RIO (("do_create() %s mode %d\n",
+				gnome_vfs_uri_to_string (uri, 0), mode));
 
 
-	flags |= O_CREAT;
-	if (exclusive)
-		flags |= O_EXCL;
-	
-	res = lookup_uri (uri, &file, &path_remainder, &connection, TRUE);
-
-	if (res) {
-		UNLOCK_SAMBA ();
-		if (res == GNOME_VFS_ERROR_NOT_FOUND)
-			res = GNOME_VFS_ERROR_ACCESS_DENIED;
-		return res;
-	}
-
-	if (file->virtual_type == SMB_VIRTUAL_TYPE_DIRECTORY) {
-		UNLOCK_SAMBA ();
-		return GNOME_VFS_ERROR_IS_DIRECTORY;
-	}
-
-	if (file->virtual_type == SMB_VIRTUAL_TYPE_FILE) {
-		UNLOCK_SAMBA ();
-		return GNOME_VFS_ERROR_ACCESS_DENIED;
-	}
-	
-	/* SMB_VIRTUAL_TYPE_SHARE */
-	dos_filename = unix_filename_to_dos ((path_remainder)?path_remainder:"/");
-	DEBUG_SMB (("tryping to cli_open '%s', flags: %d\n", dos_filename, flags));
-	fnum = cli_open (connection->cli, dos_filename, flags, DENY_NONE);
-	g_free (path_remainder);
-	g_free (dos_filename);
-	if (fnum == -1) {
-		res = gnome_vfs_result_from_cli (connection->cli);
-		smb_connection_unref (connection);
-		UNLOCK_SAMBA();
-		return res;
-	}
-	
-	handle = g_new (SmbHandle, 1);
-	handle->is_data = FALSE;
-	handle->connection = connection;
-	handle->fnum = fnum;
-	handle->offset = 0;
-
-	UNLOCK_SAMBA();
-
-	*method_handle = (GnomeVFSMethodHandle *)handle;
-#endif
-	return GNOME_VFS_OK;
+	return do_open (method, method_handle, uri, GNOME_VFS_OPEN_WRITE,
+			context);
 }
 
 static GnomeVFSResult
@@ -964,7 +733,6 @@ do_open_directory (GnomeVFSMethod *method,
 		   GnomeVFSMethodHandle **method_handle,
 		   GnomeVFSURI *uri,
 		   GnomeVFSFileInfoOptions options,
-		   const GnomeVFSDirectoryFilter *filter,
 		   GnomeVFSContext *context)
 
 {
@@ -999,6 +767,7 @@ do_open_directory (GnomeVFSMethod *method,
 		if (card == -1)
 		{
 			list = rio->virt_content;
+			path = "/";
 		} else {
 			if (card == 1 && rio->have_flash == FALSE)
 			{
@@ -1006,26 +775,24 @@ do_open_directory (GnomeVFSMethod *method,
 				return GNOME_VFS_ERROR_NOT_FOUND;
 			}
 			if (card == 0)
+			{
 				list = rio->int_content;
-			else
+				path = "Internal";
+			} else {
 				list = rio->ext_content;
+				path = "External";
+			}
 		}
 	} else {
 		GList *content = NULL;
 		RioFolderEntry *entry = NULL;
-		
+
 		list_type_is_folder = FALSE;
-		if (card == 1 && rio->have_flash == FALSE)
-		{
-			UNLOCK_RIO();
-			return GNOME_VFS_ERROR_NOT_FOUND;
-		}
-		if (card == 0)
-			content = rio->int_content;
-		else
-			content = rio->ext_content;
+
+		content = rio_get_content_from_card (card);
 
 		entry = (RioFolderEntry *) g_list_nth_data(content, folder);
+		path = entry->name;
 		list = entry->songs;
 	}
 
@@ -1055,7 +822,9 @@ do_close_directory (GnomeVFSMethod *method,
 	if (directory_handle == NULL)
 		return GNOME_VFS_OK;
 
-	g_free(directory_handle->dirname);
+	//FIXME
+	g_free (directory_handle->dirname);
+	g_free (directory_handle);
 	DEBUG_RIO(("do_close_directory: end\n"));
 
 	return GNOME_VFS_OK;
@@ -1084,7 +853,6 @@ do_read_directory (GnomeVFSMethod *method,
 	/* . or .. directories */
 	if (dh->pos == 0 || dh->pos == 1)
 	{
-		GNOME_VFS_FILE_INFO_SET_LOCAL (file_info, FALSE);
 		file_info->valid_fields = GNOME_VFS_FILE_INFO_FIELDS_NONE |
 			GNOME_VFS_FILE_INFO_FIELDS_TYPE |
 			GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
@@ -1136,48 +904,7 @@ do_read_directory (GnomeVFSMethod *method,
 	UNLOCK_RIO();
 	return GNOME_VFS_OK;
 }
-#if 0
-static GnomeVFSResult
-do_seek (GnomeVFSMethod *method,
-	 GnomeVFSMethodHandle *method_handle,
-	 GnomeVFSSeekPosition whence,
-	 GnomeVFSFileOffset offset,
-	 GnomeVFSContext *context)
 
-{
-	SmbHandle *handle = (SmbHandle *)method_handle;
-
-	DEBUG_SMB (("do_seek() %d from %d\n", offset, whence));
-	
-	switch (whence) {
-	case GNOME_VFS_SEEK_START:
-		handle->offset = offset;
-		break;
-	case GNOME_VFS_SEEK_CURRENT:
-		handle->offset += offset;
-		break;
-	case GNOME_VFS_SEEK_END:
-		return GNOME_VFS_ERROR_NOT_SUPPORTED;
-		break;
-	}
-
-	return GNOME_VFS_OK;
-}
-
-static GnomeVFSResult
-do_tell (GnomeVFSMethod *method,
-	 GnomeVFSMethodHandle *method_handle,
-	 GnomeVFSFileOffset *offset_return)
-
-
-{
-	SmbHandle *handle = (SmbHandle *)method_handle;
-
-	*offset_return = handle->offset;
-
-	return GNOME_VFS_OK;
-}
-#endif
 static GnomeVFSResult
 do_unlink (GnomeVFSMethod *method,
 	   GnomeVFSURI *uri,
@@ -1367,19 +1094,6 @@ do_truncate_handle (GnomeVFSMethod *method,
 	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 }
 
-static GnomeVFSResult
-do_get_file_info_from_handle (GnomeVFSMethod *method,
-			      GnomeVFSMethodHandle *method_handle,
-			      GnomeVFSFileInfo *file_info,
-			      GnomeVFSFileInfoOptions options,
-			      GnomeVFSContext *context)
-
-
-{
-	DEBUG_RIO(("do_get_file_info_from_handle\n"));
-	return GNOME_VFS_ERROR_NOT_SUPPORTED;
-}
-
 static GnomeVFSResult
 do_make_directory (GnomeVFSMethod *method,
 		   GnomeVFSURI *uri,
@@ -1454,6 +1168,7 @@ do_remove_directory (GnomeVFSMethod *method,
 {
 	GnomeVFSResult res;
 	gint card, folder, song;
+	GList *list;
 
 	res = lookup_uri(uri, &card, &folder, &song);
 	DEBUG_RIO(("do_remove_directory(): lookup %d %d %d\n",
@@ -1486,7 +1201,12 @@ do_remove_directory (GnomeVFSMethod *method,
 		return GNOME_VFS_ERROR_IO;
 	}
 	rio_set_card(rio->rio_hw, card);
-	res = rio_to_vfs_error(rio_del_folder(rio->rio_hw, folder));
+	list = rio_get_content_from_card (card);
+	/* Format the card when deleting the last directory */
+	if (g_list_length (list) == 1)
+		res  = rio_to_vfs_error(rio_format(rio->rio_hw));
+	else
+		res = rio_to_vfs_error(rio_del_folder(rio->rio_hw, folder));
 	rio_delete(rio->rio_hw);
 
 	if (res == GNOME_VFS_OK)
@@ -1600,57 +1320,21 @@ do_set_file_info (GnomeVFSMethod *method,
 	return res;
 }
 
-static GnomeVFSResult
-do_truncate (GnomeVFSMethod *method,
-	     GnomeVFSURI *uri,
-	     GnomeVFSFileSize where,
-	     GnomeVFSContext *context)
-
-{
-	DEBUG_RIO(("do_truncate\n"));
-	return GNOME_VFS_ERROR_NOT_SUPPORTED;
-}
-
-static GnomeVFSResult
-do_find_directory (GnomeVFSMethod *method,
-		   GnomeVFSURI *near_uri,
-		   GnomeVFSFindDirectoryKind kind,
-		   GnomeVFSURI **result_uri,
-		   gboolean create_if_needed,
-		   gboolean find_if_needed,
-		   guint permissions,
-		   GnomeVFSContext *context)
-{
-	DEBUG_RIO(("do_find_directory\n"));
-	return GNOME_VFS_ERROR_NOT_SUPPORTED;
-}
-
-static GnomeVFSResult
-do_create_symbolic_link (GnomeVFSMethod *method,
-			 GnomeVFSURI *uri,
-			 const char *target_reference,
-			 GnomeVFSContext *context)
-{
-	DEBUG_RIO(("do_create_symbolic_link\n"));
-	return GNOME_VFS_ERROR_NOT_SUPPORTED;
-}
-
-
 static GnomeVFSMethod method = {
 	sizeof (GnomeVFSMethod),
-	do_open, //FIXME
-	NULL, //do_create,
-	NULL, //do_close, //FIXME
-	NULL, //do_read, //FIXME
-	NULL, //do_write,
-	NULL, //do_seek,
-	NULL, //do_tell,
+	do_open,
+	do_create,
+	do_close,
+	do_read,
+	do_write,
+	NULL, /* do_seek */
+	NULL, /* do_tell */
 	do_truncate_handle,
 	do_open_directory,
 	do_close_directory,
 	do_read_directory,
 	do_get_file_info,
-	NULL, //do_get_file_info_from_handle,
+	NULL, /* do_get_file_info_from_handle */
 	do_is_local,
 	do_make_directory,
 	do_remove_directory,
@@ -1658,9 +1342,9 @@ static GnomeVFSMethod method = {
 	do_unlink,
 	do_check_same_fs,
 	do_set_file_info,
-	do_truncate,
-	do_find_directory,
-	do_create_symbolic_link
+	NULL, /* do_truncate */
+	NULL, /* do_find_directory */
+	NULL  /* do_create_symbolic_link */
 };
 
 GnomeVFSMethod *
@@ -1670,6 +1354,9 @@ vfs_module_init (const char *method_name, const char *args)
 
 	DEBUG_RIO (("<-- rio500 module init called -->\n"));
 	
+	/* Make sure we don't have a dangling temp file */
+	unlink ("/tmp/rio500_tmp_file.mp3");
+	
 	return &method;
 }
 
@@ -1678,6 +1365,8 @@ vfs_module_shutdown (GnomeVFSMethod *method)
 {
 	DEBUG_RIO (("<-- rio500 module shutdown called -->\n"));
 
+	/* Destroy the temp file */
+	unlink ("/tmp/rio500_tmp_file.mp3");
 	destroy_rio(rio);
 	g_mutex_free(rio_lock);
 }
